<?php
/*
 * @package     RadicalMart Package
 * @subpackage  plg_radicalmart_payment_standard
 * @version     1.0.0
 * @author      Delo Design - delo-design.ru
 * @copyright   Copyright (c) 2021 Delo Design. All rights reserved.
 * @license     GNU/GPL license: https://www.gnu.org/copyleft/gpl.html
 * @link        https://delo-design.ru/
 */

defined('_JEXEC') or die;

use Joomla\CMS\Application\CMSApplication;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Plugin\CMSPlugin;

class plgRadicalMart_PaymentStandard extends CMSPlugin
{
	/**
	 * Loads the application object.
	 *
	 * @var  CMSApplication
	 *
	 * @since  0.0.0
	 */
	protected $app = null;

	/**
	 * Affects constructor behavior.
	 *
	 * @var  boolean
	 *
	 * @since  0.0.0
	 */
	protected $autoloadLanguage = true;

	/**
	 * Prepare prices data.
	 *
	 * @param   string  $context  Context selector string.
	 * @param   object  $objData  Input data.
	 * @param   Form    $form     Joomla Form object.
	 *
	 * @throws  Exception
	 *
	 * @since  0.0.0
	 */
	public function onContentNormaliseRequestData($context, $objData, $form)
	{
		if ($context === 'com_radicalmart.shippingmethod')
		{
			JLoader::register('RadicalMartHelperPrice', JPATH_ADMINISTRATOR . '/components/com_radicalmart/helpers/price.php');
			foreach ($objData->prices as &$price)
			{
				$price['fee'] = RadicalMartHelperPrice::cleanAdjustmentValue($price['fee']);
			}
		}
	}

	/**
	 * Prepare order shipping method data.
	 *
	 * @param   string  $context   Context selector string.
	 * @param   object  $method    Method data.
	 * @param   array   $formData  Order form data.
	 * @param   array   $products  Order products data.
	 * @param   array   $currency  Order currency data.
	 *
	 * @throws  Exception
	 *
	 * @since  0.0.0
	 */
	public function onRadicalMartGetPaymentMethods($context, $method, $formData, $products, $currency)
	{
		// Set disabled
		$method->disabled = false;

		// Set price
		if (!empty($formData['payment']['price'])) $price = $formData['payment']['price'];
		else $price = (isset($method->prices[$currency['group']])) ? $method->prices[$currency['group']] : array('fee' => 0);
		$price = $this->preparePrice($price, $currency['code']);

		// Set order
		$method->order              = new stdClass();
		$method->order->id          = $method->id;
		$method->order->title       = $method->title;
		$method->order->code        = $method->code;
		$method->order->description = $method->description;
		$method->order->price       = $price;
	}

	/**
	 * Prepare order totals.
	 *
	 * @param   string  $context   Context selector string.
	 * @param   array   $total     Order total data.
	 * @param   array   $formData  Form data array.
	 * @param   object  $shipping  Shipping method data.
	 * @param   object  $payment   Payment method data.
	 * @param   array   $currency  Order currency data.
	 *
	 * @throws Exception
	 *
	 * @since 0.0.0
	 */
	public function onRadicalMartGetOrderTotal($context, &$total, $formData, $shipping, $payment, $currency)
	{
		if (!empty($payment->order->price['fee']))
		{
			$fee = RadicalMartHelperPrice::fromString($payment->order->price['fee'], $currency['code']);
			if (stripos($payment->order->price['fee'], '%') !== false) $fee = $total['final'] * ($fee / 100);

			if ($fee)
			{
				$price = $this->preparePrice(array('fee' => $payment->order->price['fee']), $currency['code']);
				foreach ($price as $key => $value) $total[$key] = $value;
				$total['final'] = $total['final'] + $fee;
			}
		}
	}

	/**
	 * Prepare order form.
	 *
	 * @param   string  $context   Context selector string.
	 * @param   Form    $form      Order form object.
	 * @param   array   $data      Form data array.
	 * @param   object  $shipping  Shipping method data.
	 * @param   object  $payment   Payment method data.
	 *
	 * @since 0.0.0
	 */
	public function onRadicalMartGetOrderForm($context, $form, $data, $shipping, $payment)
	{
		if (!empty($payment->order->price['fee']))
		{
			$form->setFieldAttribute('fee', 'default', $payment->order->price['fee'], 'payment.price');
		}
	}

	/**
	 * Prepare price values.
	 *
	 * @param   array   $price  Item price array.
	 * @param   string  $code   Currency code.
	 *
	 * @throws Exception
	 *
	 * @return array Formatting price array, False on failure.
	 *
	 * @since  0.0.0
	 */
	protected function preparePrice($price = array(), $code = null)
	{
		// Set fee price
		$price['fee'] = RadicalMartHelperPrice::cleanAdjustmentValue($price['fee']);
		if (stripos($price['fee'], '%') !== false)
		{
			$price['fee_string'] = $price['fee'];
			$price['fee_seo']    = htmlspecialchars($price['fee']);
			$price['fee_number'] = null;
		}
		else
		{
			$price['fee']        = RadicalMartHelperPrice::clean($price['fee'], $code);
			$price['fee_string'] = (!empty($price['fee'])) ?
				RadicalMartHelperPrice::toString($price['fee'], $code) : null;
			$price['fee_seo']    = (!empty($price['fee'])) ?
				RadicalMartHelperPrice::toString($price['fee'], $code, 'seo') : null;
			$price['fee_number'] = (!empty($price['fee'])) ?
				RadicalMartHelperPrice::toString($price['fee'], $code, false) : null;
		}

		return $price;
	}
}